
------------------------------------------------------------------------
--
-- ShieldFrames
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


local ShieldFrames = _G.ShieldFrames;
if(not ShieldFrames) then return; end

local HeaderProto = {};
ShieldFrames.HeaderProto = HeaderProto;

local HeaderFrameCounter = 1;
local HeaderCache = {};


function HeaderProto:New(anchor)
	if(#HeaderCache > 0) then
		local cachedHeader = tremove(HeaderCache, 1);
		
		cachedHeader:ApplyProfile();
		
		return cachedHeader;
	end

	local o = setmetatable({}, self);
	self.__index = self;
	
	o.anchor = anchor;
	o.profile = anchor.profile;
	
	o:CreateFrame();
	
	return o;
end

function HeaderProto:Release()
	self.frame:Hide();
	
	self.frame:ClearAllPoints();
	
	tinsert(HeaderCache, self);
end

function HeaderProto:CreateFrame()
	local frame = CreateFrame("Frame", "SF_Header"..HeaderFrameCounter, UIParent);
	self.frame = frame;
	
	HeaderFrameCounter = HeaderFrameCounter + 1;
	
	local leftText = frame:CreateFontString(nil, "OVERLAY");
	self.leftText = leftText;
	
	leftText:SetFontObject(GameFontHighlightSmallOutline);
	leftText:SetPoint("LEFT", frame, "LEFT", 4, 0);
	
	local rightText = frame:CreateFontString(nil, "OVERLAY");
	self.rightText = rightText;
	
	rightText:SetFontObject(GameFontHighlightSmallOutline);
	rightText:SetPoint("RIGHT", frame, "RIGHT", -4, 0);
	
	self:ApplyProfile();
end

function HeaderProto:ApplyProfile()
	local profile = self.profile;
	local frame = self.frame;

	frame:SetScale(profile.scale);
	frame:SetWidth(profile.width);
	frame:SetHeight(profile.headerHeight);
	frame:SetAlpha(profile.headerAlpha);
	
	local path, size, style = GameFontHighlightSmallOutline:GetFont();
	size = size + profile.headerTextSize;	
	
	self.leftText:SetFont(path, size, style);
	self.rightText:SetFont(path, size, style);
	
	self.frame:Hide();
	
	if(profile.headerVisible) then
		self.frame:Show();
	end
end

-- see EffectProto:AnchorTo for details
function HeaderProto:AlignTo(alignObject, reverseGrowth)
	local profile = self.profile;
	
	self.frame:ClearAllPoints();

	if(reverseGrowth) then
		self.frame:SetPoint("BOTTOMLEFT", alignObject.frame, "TOPLEFT", 0, profile.headerSpacing);
	else
		self.frame:SetPoint("TOPLEFT", alignObject.frame, "BOTTOMLEFT", 0, -1 * profile.headerSpacing);		
	end
end
