
------------------------------------------------------------------------
--
-- ShieldFrames
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


local ShieldFrames = _G.ShieldFrames;

local AceDialog = nil;
local AceRegistry = nil;

local options;


-- #################################################################################################


local function getGeneralOption(info)
	return ShieldFrames.db.profile[info.arg];
end

local function setGeneralOption(info, value)
	ShieldFrames.db.profile[info.arg] = value;
end

local function getAnchorOption(info)	
	return ShieldFrames.db.profile.anchors[info[2]][info.arg];
end

local function setAnchorOption(info, value)
	local anchorName = info[2];	
	
	ShieldFrames.db.profile.anchors[anchorName][info.arg] = value;
	ShieldFrames.Anchors[anchorName]:ApplyProfile();	
end

local function getRelativeUnitFilterOption(info)
	return ShieldFrames.db.profile.anchors[info[2]].unitRelativeFilter[info.arg];
end

local function setRelativeUnitFilterOption(info, value)
	local anchorName = info[2];
	
	ShieldFrames.db.profile.anchors[anchorName].unitRelativeFilter[info.arg] = value and value or nil;
	ShieldFrames.Anchors[anchorName]:ApplyFilter();
end


-- #################################################################################################


function options_AnchorProto(name)
	return {
		type = "group",
		childGroups = "tab",		
		name = name,
		get = getAnchorOption,
		set = setAnchorOption,
		
		args = {
			general = {
				type = "group",
				name = "General",
				order = 1,
				
				args = {
					name = {
						type = "input",
						name = "Rename",
						order = 1,
						get = function(info) return info[2]; end,
						set = function(info, value)
							local oldName = info[2];
							
							AceDialog:SelectGroup("ShieldFrames", "anchors");
							
							ShieldFrames:RenameFrameAnchor(oldName, value);						
							
							local anchorGroup = info.options.args.anchors.args;
							
							anchorGroup[value] = anchorGroup[oldName];
							anchorGroup[value].name = value;
							
							anchorGroup[oldName] = nil;
							
							AceRegistry:NotifyChange("ShieldFrames");
							
							AceDialog:SelectGroup("ShieldFrames", "anchors", value);
						end,
					},
					delete = {
						type = "execute",
						name = "Delete",
						order = 2,
						confirm = function() return "Are you sure you want to delete this frame anchor?" end,			
						func = function(info)
							local name = info[2];
						
							AceDialog:SelectGroup("ShieldFrames", "anchors");
							
							ShieldFrames:DeleteFrameAnchor(name);
							
							info.options.args.anchors.args[name] = nil;
							
							AceRegistry:NotifyChange("ShieldFrames");
						end,
					},
					type = {
						type = "group",
						name = "Type",
						inline = true,
						order = 3,						
						
						args = {
							sep1 = {
								type = "description",						
								name = "The anchor type defines how the different effects will be grouped and sorted. Please note that changing this value causes the anchor to be cleared off all effects.",
								order = 1,
								width = "full",
								hidden = false,
							},
							type = {
								type = "select",
								name = "",
								order = 2,
								arg = "type",
								values = {
									["ungrouped"] = "Ungrouped",
									["byUnit"] = "By Unit",
								},
								set = function(info, value)
									local anchor = ShieldFrames.Anchors[info[2]];
								
									anchor:Clear();								
									setAnchorOption(info, value);
									anchor:SetupType();
								end
							},
						},
					},
				},
			},
			frame = {
				type = "group",
				name = "Frames",
				order = 2,
				
				args = {	
					scale = {
						type = "range",
						name = "Scale",
						order = 1,
						min = 0.1, max = 2.0, step = 0.01,
						arg = "scale",
					},
					width = {
						type = "range",
						name = "Width",
						order = 2,
						min = 10, max = 1000, step = 1,
						arg = "width",
					},				
					anchor = {
						type = "group",
						name = "Anchor",
						inline = true,
						order = 4,
						
						args = {
							visible = {
								type = "toggle",
								name = "Show Anchor",
								order = 1,
								arg = "anchorVisible",
							},						
							sep1 = {
								type = "description",						
								name = "",
								order = 2,
								width = "full",
								hidden = false,
							},
							posX = {
								type = "range",
								name = "X offset from left",
								order = 3,
								min = 0, max = floor(GetScreenWidth()), step = 0.1,
								arg = "posX",
							},
							posY = {
								type = "range",
								name = "Y offset from bottom",
								order = 4,
								min = 0, max = floor(GetScreenHeight()), step = 0.1,
								arg = "posY",
							},
							sep2 = {
								type = "description",						
								name = "",
								order = 5,
								width = "full",
								hidden = false,
							},					
							height = {
								type = "range",
								name = "Anchor Height",
								order = 6,
								min = 1, max = 100, step = 1,
								arg = "anchorHeight",
							},	
							alpha = {
								type = "range",
								name = "Anchor Alpha",
								order = 7,
								min = 0.0, max = 1.0, step = 0.05,
								arg = "anchorAlpha",
							},		
						},
					},
					type_ungrouped = {
						type = "group",
						name = "Type-specific - Ungrouped",
						inline = true,
						hidden = function(info)
							return (ShieldFrames.db.profile.anchors[info[2]].type ~= "ungrouped");
						end,
						order = 5,
						
						args = {
							growth = {
								type = "toggle",
								name = "Reverse Growth",
								desc = "Add new bars on top of previous bars",
								order = 1,
								arg = "reverseGrowth",
								set = function(info, value)
									setAnchorOption(info, value);
									
									ShieldFrames.Anchors[info[2]]:layout_ReanchorFrames();	
								end,
							},				
						},
					},
					
					type_grouped = {
						type = "group",
						name = "Type-specific - Grouped",
						inline = true,
						hidden = function(info)
							return (ShieldFrames.db.profile.anchors[info[2]].type == "ungrouped");
						end,
						order = 5,
						
						args = {
							visible = {
								type = "toggle",
								name = "Show Header",
								order = 1,
								arg = "headerVisible",
							},						
							sep1 = {
								type = "description",						
								name = "",
								order = 2,
								width = "full",
								hidden = false,
							},
							height = {
								type = "range",
								name = "Header Height",
								order = 3,
								min = 1, max = 100, step = 1,
								arg = "headerHeight",
							},	
							alpha = {
								type = "range",
								name = "Header Alpha",
								order = 4,
								min = 0.0, max = 1.0, step = 0.05,
								arg = "headerAlpha",
							},							
							sep2 = {
								type = "description",						
								name = "",
								order = 5,
								width = "full",
								hidden = false,
							},
							textSize = {
								type = "range",
								name = "Header Relative Text Size",
								order = 6,
								min = -10, max = 10, step = 1,
								arg = "headerTextSize",
							},
							spacing = {
								type = "range",
								name = "Header Spacing",
								order = 7,
								min = 1, max = 20, step = 1,
								arg = "headerSpacing",
								set = function(info, value)
									setAnchorOption(info, value);
									
									ShieldFrames.Anchors[info[2]]:layout_ReanchorFrames();	
								end								
							},
								
						},
					},
					effects = {
						type = "group",
						name = "Effects",
						inline = true,
						order = 6,
						
						args = {
							iconVisible = {
								type = "toggle",
								name = "Show Icon",
								order = 1,
								arg = "iconVisible",
							},
							durationVisible = {
								type = "toggle",
								name = "Show Duration",
								order = 2,
								arg = "durationVisible",
							},
							sep1 = {
								type = "description",						
								name = "",
								order = 3,
								width = "full",
								hidden = false,
							},								
							effectHeight = {
								type = "range",
								name = "Effect Height",
								order = 4,
								min = 1, max = 100, step = 1,
								arg = "effectHeight",
							},
							durationHeight = {
								type = "range",
								name = "Duration Height",
								order = 5,
								min = 1, max = 100, step = 1,
								arg = "durationHeight",
							},
							sep2 = {
								type = "description",						
								name = "",
								order = 6,
								width = "full",
								hidden = false,
							},					
							effectAlpha = {
								type = "range",
								name = "Effect Alpha",
								order = 7,
								min = 0.0, max = 1.0, step = 0.05,
								arg = "effectAlpha",
							},
							effectSpacing = {
								type = "range",
								name = "Effect Spacing",
								order = 8,
								min = 1, max = 20, step = 1,
								arg = "effectSpacing",
								set = function(info, value)
									setAnchorOption(info, value);
									
									ShieldFrames.Anchors[info[2]]:layout_ReanchorFrames();	
								end
							},				
						},						
					},
					text = {
						type = "group",
						name = "Text",
						inline = true,
						order = 7,
						
						args = {
							sep1 = {
								type = "description",						
								name = "Here you can customize the data displayed in the effect bar itself. You can use the following symbols:\nsource name %s, dest name %d, spell name %n, current absorb amount %a, maximum absorb amout %m, current percentage %p",
								order = 1,
								width = "full",
								hidden = false,
							},					
							left = {
								type = "input",
								name = "Left Text",
								order = 2,
								arg = "leftFormat",
							},	
							center = {
								type = "input",
								name = "Center Text",
								order = 3,
								arg = "centerFormat",
							},
							right = {
								type = "input",
								name = "Right Text",
								order = 4,
								arg = "rightFormat",
							},
							size = {
								type = "range",
								name = "Relative Text Size",
								order = 5,
								min = -10, max = 10, step = 1,
								arg = "effectTextSize",
							},					
						},
					},
				},
			},
			simpleFilter = {
				type = "group",
				name = "Basic Filter",
				order = 3,
				
				args = {		
					sep1 = {
						type = "description",
						name = "Please note that any settings you change here will only take effect for new effects, not the one being displayed at the moment.",
						order = 1,
						width = "full",
						hidden = false,
					},	
					sep2 = {
						type = "description",
						name = "These are very basic filters for common usage and will be sufficient for most use cases. You can further customize the filters on the Unit Filter and Spell Filter tab to any special needs.",
						order = 2,
						width = "full",
						hidden = false,
					},
					appliedByMe = {
						type = "toggle",
						name = "Show only my effects",
						order = 3,
						arg = "filterAppliedByMe",
					},	
					appliedOnMe = {
						type = "toggle",
						name = "Show only effects on me",
						desc = "THIS OPTION OVERRULES ANY UNIT FILTER SETTING YOU MADE",
						order = 4,
						arg = "filterAppliedOnMe",
					},					
				},
			},
			
			unitFilter = {
				type = "group",
				name = "Unit Filter",
				disabled = function(info)
					return ShieldFrames.db.profile.anchors[info[2]].filterAppliedOnMe;
				end,
				order = 4,
				
				args = {
					sep1 = {
						type = "description",
						name = "Please note that any settings you change here will only take effect for new effects, not the one being displayed at the moment.",
						order = 1,
						width = "full",
						hidden = false,
					},				
					mode = {
						type = "group",
						name = "Filter Mode",
						inline = true,
						order = 2,
						
						args = {
							sep1 = {
								type = "description",
								name = "An inclusive filter will only show the items selected here and nothing else, while an exclusive filter will show everything except the items selected here. Try to always use the type for which you need to select the least amount of items for optimal performance.",
								order = 1,
								width = "full",
								hidden = false,
							},
							mode = {						
								type = "select",
								name = "",
								order = 2,
								arg = "unitFilterMode",
								values = {
									["inclusive"] = "Inclusive",
									["exclusive"] = "Exclusive",
								},
								set = function(info, value)
									setAnchorOption(info, value);
									
									ShieldFrames.Anchors[info[2]]:ApplyFilter();
								end
							},
						},
					},		
					-- No multiselect here to have a proper ordering and meaningful descriptions			
					relativeUnits = {
						type = "group",
						name = "Relative Units",
						inline = true,
						order = 3,
						get = getRelativeUnitFilterOption,
						set = setRelativeUnitFilterOption,
												
						args = {
							player = {
								type = "toggle",
								name = "Player",
								order = 1,
								arg = "player",
							},
							target = {
								type = "toggle",
								name = "Target",
								order = 2,
								arg = "target",
							},
							focus = {
								type = "toggle",
								name = "Focus",
								order = 3,
								arg = "focus",
							},
							sep1 = {
								type = "description",
								name = "",
								order = 4,
								width = "full",
								hidden = false,
							},
							party1 = {
								type = "toggle",
								name = "Party 1",
								order = 5,
								arg = "party1",
							},
							party2 = {
								type = "toggle",
								name = "Party 2",
								order = 6,
								arg = "party2",
							},
							party3 = {
								type = "toggle",
								name = "Party 3",
								order = 7,
								arg = "party3",
							},
							party4 = {
								type = "toggle",
								name = "Party 4",
								order = 8,
								arg = "party4",
							},							
						},
					},					
					specificUnits = {
						type = "group",
						name = "Specific Units",						
						inline = true,
						order = 4,
						
						args = {							
						},
					},
				},
			},			
			spellFilter = {
				type = "group",
				name = "Spell Filter",
				order = 5,
				
				args = {
					sep1 = {
						type = "description",
						name = "Please note that any settings you change here will only take effect for new effects, not the one being displayed at the moment.",
						order = 1,
						width = "full",
						hidden = false,
					},
					mode = {
						type = "group",
						name = "Filter Mode",
						inline = true,
						order = 2,
						
						args = {
							sep1 = {
								type = "description",
								name = "An inclusive filter will only show the items selected here and nothing else, while an exclusive filter will show everything except the items selected here. Try to always use the type for which you need to select the least amount of items for optimal performance.",
								order = 1,
								width = "full",
								hidden = false,
							},
							mode = {						
								type = "select",
								name = "",
								order = 2,
								arg = "unitFilterMode",
								values = {
									["inclusive"] = "Inclusive",
									["exclusive"] = "Exclusive",
								},
							},
						},						
					},		
					sep2 = {
						type = "description",
						name = "Sorry, but this filter will only be available in the Cataclysm version of this addon",
						order = 3,
						width = "full",
						hidden = false,
					},								
				},
			},
		},
	};
end

local options = {
	type = "group",
	get = getGeneralOption,
	set = setGeneralOption,
	
	args = {
		general = {
			type = "group",
			name = "General Options",
			cmdInline = true,
			order = 1,
			
			args = {	
				enabled = {
					type = "toggle",
					order = 1,
					name = "Enabled",
					desc = "Enables/Disables this addon",
					get = function() return ShieldFrames:IsEnabled() end,
					set = function()
						if(ShieldFrames:IsEnabled()) then
							ShieldFrames:Disable();
						else
							ShieldFrames:Enable()
						end
					end,
				},
				locked = {
					type = "toggle",
					order = 2,
					name = "Lock Frames",
					desc = "Locks/Unlocks the frame anchors for repositioning",
					get = function() return ShieldFrames.isLocked end,
					set = function()
						if(ShieldFrames.isLocked) then
							ShieldFrames:UnlockAnchors();
						else
							ShieldFrames:LockAnchors();
						end
					end,
				},
				sep1 = {
					type = "description",
					name = "",
					width = "full",
					hidden = false,
					order = 3,
				},
				test = {
					type = "execute",
					name = "Appy Test Effects",
					desc = "Please wait for them to run out before applying new ones",
					order = 4,
					func = function()
						ShieldFrames:Test();
					end,
				},
			},
		},
		
		anchors = {
			type = "group",
			name = "Frame Anchors",
			order = 2,
			
			args = {
				__add = {
					type = "group",
					inline = true,
					name = "Add new frame anchor",
					
					args = {
						desc = {
							type = "description",							
							name = "A frame anchor serves as the spawnpoint and controller of effect bars. Every anchor can display, group or sort its effect bars differently, and can filter for specific units and effects.",
							order = 1,
							width = "full",
							hidden = false,
						},
						input = {
							type = "input",
							name = "Anchor name",
							order = 2,
							validate = function(info, value)
								if(value == "__add") then
									return "This anchor name is reserved for internal use";
								end
							
								if(ShieldFrames.Anchors[value]) then
									return "There is already a frame anchor with that name";
								end
								
								return true;
							end,
							set = function(info, value)
								ShieldFrames:CreateFrameAnchor(value);
								
								info.options.args.anchors.args[value] = options_AnchorProto(value);
								
								AceRegistry:NotifyChange("ShieldFrames");
							end,
						},
					},
				},
			},
		}
	},
};


-- #################################################################################################
	

local function LoadAnchors()
	local anchors = ShieldFrames.db.profile.anchors;
	local anchorOptions = options.args.anchors.args;
	
	for name, anchor in pairs(anchors) do
		anchorOptions[name] = options_AnchorProto(name);
	end
end

function ShieldFrames:ShowOptions()
	if(not AceDialog) then
		AceDialog = LibStub("AceConfigDialog-3.0");
		AceRegistry = LibStub("AceConfigRegistry-3.0");
		
		LoadAnchors();
		
		LibStub("AceConfig-3.0"):RegisterOptionsTable("ShieldFrames", options);
		AceDialog:SetDefaultSize("ShieldFrames", 835, 525);
	end
	
	AceDialog:Open("ShieldFrames");
end

function ShieldFrames:TestConfig()
	AceDialog:SelectGroup("ShieldFrames", "anchors", "Default");
end
